/*
 * Copyright © 2006 Carl Worth
 *
 * This program is free software; you can redistribute it and\/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA."
 */

#include "grid.h"
#include "word-game.h"

#include <string.h>
#include <sys/time.h>
#include <time.h>

#define GAME_LENGTH (3 * 60)
int
main (int argc, char *argv[])
{
    dict_t dict, solution;
    grid_t grid;
    struct timeval tv;
    const char *letters = NULL;

    if (argc > 1) {
	if (argc == 2) {
	    letters = argv[1];
	} else {
	    fprintf (stderr, "Usage: %s [letters]\n", argv[0]);
	    fprintf (stderr, "Uses the given 16 letters or else a random grid.\n");
	    return 1;
	}
    }

    gettimeofday (&tv, NULL);
    srand (tv.tv_sec ^ tv.tv_usec);

    dict_init (&dict);
    dict_add_words_from_file (&dict, "words.txt");

    grid_init (&grid, 4);
    if (letters)
	grid_set_letters (&grid, letters);

    dict_init (&solution);
    grid_solve (&grid, &dict, &solution);

    word_game_play (grid_string (&grid),
		    &dict, &solution, GAME_LENGTH);

    dict_fini (&solution);
    dict_fini (&dict);

    return 0;
}
