/*
 * Copyright © 2006 Carl Worth
 *
 * This program is free software; you can redistribute it and\/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA."
 */
#include <stdlib.h>
#include <string.h>
#include <goocanvas.h>
#include <sys/time.h>
#include <time.h>
#include <ctype.h>
#include <math.h>

#include "word-game.h"
#include "demo-item.h"

static gboolean
on_delete_event (GtkWidget *window,
		 GdkEvent  *event,
		 gpointer   unused_data)
{
    exit (0);
}

static GtkWidget *
create_window (void)
{
    GtkWidget *window, *scrolled_window;

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_default_size (GTK_WINDOW (window), 500, 500);
    gtk_widget_show (window);
    g_signal_connect (window, "delete_event",
		      (GtkSignalFunc) on_delete_event, NULL);

    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
					 GTK_SHADOW_IN);
    gtk_scrolled_window_set_policy  (GTK_SCROLLED_WINDOW (scrolled_window),
				     GTK_POLICY_AUTOMATIC,
				     GTK_POLICY_AUTOMATIC);
    gtk_widget_show (scrolled_window);
    gtk_container_add (GTK_CONTAINER (window), scrolled_window);

    return scrolled_window;
}

#define MAX_LETTERS 7
static GooCanvasItem *letter_items[MAX_LETTERS];
static int num_letters;

static int
rand_within (int num_values)
{
    return (int) ((double) num_values * (rand() / (RAND_MAX + 1.0)));
}

static void
shuffle (int *array, int length)
{
    int i, r, tmp;

    for (i = 0; i < length; i++)
    {
	r = i + rand_within (length - i);
	tmp = array[i];
	array[i] = array[r];
	array[r] = tmp;
    }
}

#define LETTER_SIZE 60
#define LETTER_PAD 5

static void
get_letter_position (int i, int *x, int *y)
{
    *x = 20 + i * (LETTER_SIZE + LETTER_PAD);
    *y = 20;
}

static gboolean
on_button_press (GooCanvasItem  *item,
		 GooCanvasItem  *target,
		 GdkEventButton *event,
		 gpointer        data)
{
    int indices[MAX_LETTERS];
    int i, x, y;

    for (i = 0; i < num_letters; i++)
	indices[i] = i;

    shuffle (indices, num_letters);

    for (i = 0; i < num_letters; i++) {
	get_letter_position (indices[i], &x, &y);
	goo_demo_item_move_to (letter_items[i], x, y);
    }

    return TRUE;
}

static void
create_canvas (GtkWidget *parent, char *word)
{
    GtkWidget *canvas;
    GooCanvasItem *root;
    int i;
    int x, y;

    canvas = goo_canvas_new ();
    gtk_widget_set_size_request (canvas, 400, 400);
    goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 400, 400);
    gtk_widget_show (canvas);
    gtk_container_add (GTK_CONTAINER (parent), canvas);

    root = goo_canvas_get_root_item (GOO_CANVAS (canvas));

    for (i = 0; i < MIN (MAX_LETTERS, strlen (word)); i++) {
	get_letter_position (i, &x, &y);
	letter_items[i] = goo_demo_item_new (root,
					     x, y,
					     LETTER_SIZE,
					     word[i],
					     NULL);

	g_signal_connect (letter_items[i], "button_press_event",
			  (GtkSignalFunc) on_button_press, NULL);
    }
    num_letters = i;
    while (i < MAX_LETTERS)
	letter_items[i] = NULL; 
}

int
main (int argc, char *argv[])
{
    struct timeval tv;
    bag_t bag;
    char rack[8];
    int i;
    GtkWidget *window;

    gettimeofday (&tv, NULL);
    srand (tv.tv_sec ^ tv.tv_usec);

    bag_init (&bag);
    bag_shuffle (&bag);

    memcpy (rack, bag.tiles, 7);
    rack[7] = '\0';

    for (i = 0; i < 7; i++)
	rack[i] = toupper (rack[i]);

    gtk_init (&argc, &argv);
    window = create_window ();

    create_canvas (window, rack);

    gtk_main ();

    return 0;
}
